
package modelage;

import java.awt.Graphics;
import java.util.Arrays;

/**
 * Class which governs what is shown on the panel 40Ar/40K data plot
 */

public class ResultsPanel extends javax.swing.JPanel{
	private static final long serialVersionUID = 1L;
	
	public static int middle = 0;
	public static int geneticPerformed = 0;
	public static boolean plotAge = false;
	public static boolean plotArK = false;
	
	public static double diagArK = 0;
	public static double detrArK = 0;	
	public static double ageDiagFinal = 0;
	public static double ageDetrFinal = 0;

	public static double[] arKInitialValues;
	public static double[] arKModifiedValues;
	public static double[] wtInitialValues;
	public static double[] wtModifiedValues;
		
	
	
	private final int leftOffset = 40;
	private final int rightOffset = 40;
	private final int upperOffset = 40;
	private final int lowerOffset = 40;
	private int noOfPoints =6;	
	
	double dividend;
	
	private int maxWidth;
	private int maxHeight;
	private int frameWidth;
	private int frameHeight;
	
	
	public ResultsPanel() {
		 super();
	}

	
	/**
	 * Governs what is shown on the panel
	 */
	public void paintComponent(final Graphics g) {
		super.paintComponent(g);

		maxWidth = getWidth();
		maxHeight = getHeight();
		frameWidth = Math.abs(maxWidth-rightOffset - leftOffset)+1;
		frameHeight = Math.abs(upperOffset - maxHeight + lowerOffset);

	//frame
		g.setColor(new java.awt.Color(0, 0, 0));
		drawFrame(g, maxWidth, maxHeight);
	//X subtitles
		drawXSubtitles(g, maxWidth, maxHeight);
	//Y subtitles	
		int maxAge = 0;
		if (middle != 0){
			maxAge = drawYSubtitles(g, maxWidth, maxHeight, arKInitialValues);
		}		
	//line drawing - the lines are drawn only after the calculations were performed
		if (geneticPerformed !=0){
			final double[] drawing_age = new double[101];
			for (int i=0; i <= 100; i++){
			   drawing_age[i] = (100.0 * ageDiagFinal + i * Math.abs(ageDetrFinal - ageDiagFinal))/100;
			}			
		//drawing initial values and line
			g.setColor(new java.awt.Color(160, 160, 160));
			drawPointsAndLine(g, maxWidth, maxHeight, arKInitialValues, wtInitialValues, maxAge);			
		//drawing final values and line	
			g.setColor(new java.awt.Color(0, 0, 0));
			drawPoints(g, maxWidth, maxHeight, arKModifiedValues, wtModifiedValues, maxAge);
			drawLine(g, maxWidth, maxHeight, diagArK, detrArK, maxAge);
		}
	}

    /** 
     * Set the final values of 40Ar/40K for diagenetic and detrital end members
     * 
	 */
	public static void setArKEndMemberValues (double diagArKValue, double detrArKValue){
		diagArK = diagArKValue;
		detrArK = detrArKValue;
	}
	
	/**
	 * Set the initial 40Ar/40K values 
	 */
	public static void setInitialArKValues(double[] arKValuesToSet){
		arKInitialValues = new double[arKValuesToSet.length];
		for (int i = 0; i < arKInitialValues.length; i++){
			arKInitialValues[i] = arKValuesToSet[i];
		}
	}
	/**
	 * Set the final 40Ar/40K values 
	 */
	public static void setModifiedArKValues(double[] arKValuesToSet){
		arKModifiedValues = new double[arKValuesToSet.length];
		for (int i = 0; i < arKModifiedValues.length; i++){
			arKModifiedValues[i] = arKValuesToSet[i];
		}
	}
	
	/**
	 * Set the initial Wt values
	 */
	public static void setInitialWtValues(double[] initialWtValuesToSet){
		wtInitialValues = new double[initialWtValuesToSet.length];
		for (int i = 0; i < wtInitialValues.length; i++){
			wtInitialValues[i] = initialWtValuesToSet[i];
		}
	}
	
	/**
	 * Set the final Wt (Ik) values
	 */
	public static void setModifiedWtValues(double[] modifiedWtValuesToSet){
		wtModifiedValues = new double[modifiedWtValuesToSet.length];
		for (int i = 0; i < wtModifiedValues.length; i++){
			wtModifiedValues[i] = modifiedWtValuesToSet[i];
		}
	}
	
	/**
	 * handle drawing of a line
	 */
	private void drawLine(final Graphics g, int maxWidth, int maxHeight, double arKValuesDiag, double arKValuesDetr, int maxAge){

		dividend = maxAge/noOfPoints;
		g.drawLine(leftOffset, 
				upperOffset + frameHeight - ((int)(Math.round(arKValuesDiag * frameHeight / ((double)dividend *5)))),
				maxWidth - rightOffset, 
				upperOffset + frameHeight - ((int)(Math.round(arKValuesDetr * frameHeight / ((double)dividend *5)))));
	}
	
	/**
	 * handle drawing squares and line
	 */
	private void drawPointsAndLine(final Graphics g, int maxWidth, int maxHeight, double[] arKValues, double[] initialWtValues, int maxAge){
		dividend = maxAge/noOfPoints;
		for(int i = 0; i < initialWtValues.length; i++){
			g.fillRect((leftOffset + (int)(initialWtValues[i] * frameWidth / 100)-3), 
					upperOffset + frameHeight - ((int)(Math.round((arKValues[i]) * frameHeight / ((double)dividend *5))))-3, 6, 6);
		}
		double[] linearFitResults = new double[3];
		linearFitResults = linear_fit(initialWtValues, arKValues);
		
		drawLinearFit(g, linearFitResults[0], linearFitResults[1], maxWidth, maxHeight, maxAge);
	}
	
	/**
	 * handle drawing of the points on the diagram
	 */
	private void drawPoints(final Graphics g, int maxWidth, int maxHeight, double[] arKValues, double[] initialWtValues, int maxAge){
		double dividend = maxAge/noOfPoints;
		for(int i = 0; i < initialWtValues.length; i++){
			g.fillRect((leftOffset + (int)(initialWtValues[i] * frameWidth / 100)-3), 
					upperOffset + frameHeight - ((int)(Math.round((arKValues[i]) * frameHeight / ((double)dividend *5))))-3, 6, 6);
		}
	}
	
	/**
	 * handle drawing of the line y = a + bx with parameters a and b
	 */
	private void drawLinearFit(Graphics g, double a, double b, int maxWidth, int maxHeight, int maxAge){
		double dividend = maxAge/noOfPoints;
		for (int x=0; x < 100; x++){
			   g.drawLine((leftOffset + x * frameWidth / 100), 
					   upperOffset + frameHeight - (int)(Math.round((a + b * x) * frameHeight / (dividend * 5))), 
					   (leftOffset + (x+1) * frameWidth / 100), 
					   upperOffset + frameHeight - (int)(Math.round((a + b * (x + 1)) * frameHeight / (dividend * 5))));
		}
	}

	/**
	 * handle drawing of the frame
	 */
	private void drawFrame(final Graphics g, int maxWidth, int maxHeight){
		g.setColor(new java.awt.Color(0, 0, 0));
		g.drawLine(leftOffset, upperOffset, leftOffset, maxHeight-lowerOffset);
		g.drawLine(leftOffset, maxHeight-lowerOffset, maxWidth-rightOffset, maxHeight-lowerOffset);
		g.setColor(new java.awt.Color(100, 190, 190));
		g.drawLine(leftOffset, upperOffset, maxWidth-rightOffset, upperOffset);
		g.drawLine(maxWidth-rightOffset, upperOffset, maxWidth-rightOffset, maxHeight-lowerOffset);
	}
	
	/**
	 * handle appearance of X labels
	 */
	private void drawXSubtitles(final Graphics g, int maxWidth, int maxHeight){
		int jedenOdstep = frameWidth/4;
		g.setColor(new java.awt.Color(0, 0, 0));
		g.drawLine(leftOffset, maxHeight-lowerOffset, leftOffset, maxHeight-lowerOffset+5);
		g.drawLine(leftOffset + jedenOdstep, maxHeight-lowerOffset, leftOffset + jedenOdstep, maxHeight-lowerOffset+5);
		g.drawLine(leftOffset + jedenOdstep*2, maxHeight-lowerOffset, leftOffset + jedenOdstep*2, maxHeight-lowerOffset+5);
		g.drawLine(leftOffset + jedenOdstep*3, maxHeight-lowerOffset, leftOffset + jedenOdstep*3, maxHeight-lowerOffset+5);
		g.drawLine(maxWidth-rightOffset, maxHeight-lowerOffset, maxWidth-rightOffset, maxHeight-lowerOffset+5);
		g.drawString("0", leftOffset - 2, maxHeight-lowerOffset+16);
		g.drawString("25", leftOffset + jedenOdstep - 5, maxHeight-lowerOffset+16);
		g.drawString("50", leftOffset + jedenOdstep*2 - 5, maxHeight-lowerOffset+16);
		g.drawString("75", leftOffset + jedenOdstep*3 - 5, maxHeight-lowerOffset+16);
		g.drawString("100", maxWidth-rightOffset - 8, maxHeight-lowerOffset+16);
		g.drawString("% Id", leftOffset + jedenOdstep*2 - 10, maxHeight-lowerOffset+30);;
	}
	
	/**
	 * handle appearance of Y labels
	 */
	private int drawYSubtitles(final Graphics g, int maxWidth, int maxHeight, double[] arKValues){
 		
		double[] ageValueClone = new double[arKValues.length];//table formed in order to sort it and subsequently draw diagram
		
 		for(int i=0; i<arKValues.length; i++){
			ageValueClone[i] = arKValues[i];
			}
 		Arrays.sort(ageValueClone);
 		
 		int middle = (int)(ageValueClone[0] + ageValueClone[ageValueClone.length-1])/2;
 		
 		int maxAge = middle * 3;
		double dividend = maxAge/noOfPoints;	
		int frameSize = Math.abs(maxHeight - upperOffset - lowerOffset);
		int step = (int) ((int)frameSize/(double)(noOfPoints-1));
		
	//left side letters
		for (int i = 1; i < noOfPoints; i ++){ 	
			g.drawLine(leftOffset - 5, upperOffset + ((noOfPoints - 1 - i) * step), leftOffset, upperOffset + ((noOfPoints - 1 - i) * step));
			if ((dividend * i) >= 100 && (dividend * i) < 1000){
				g.drawString((dividend * i) + "", leftOffset - 35, upperOffset + ((noOfPoints - 1 - i) * step)+3);
			}			
			if ((dividend * i) >= 10 && (dividend * i) < 100){
				g.drawString((dividend * i) + "", leftOffset - 28, upperOffset + ((noOfPoints - 1 - i) * step)+3);
			}			
			if ((dividend * i) >= 0 && (dividend * i) < 10){
				g.drawString((dividend * i) + "", leftOffset - 22, upperOffset + ((noOfPoints - 1 - i) * step)+3);
			}
		}
		g.drawLine(leftOffset - 5, upperOffset + frameSize, leftOffset, upperOffset + frameSize);
		g.drawString(0 + "", leftOffset - 14, maxHeight - lowerOffset + 3);
		
		return maxAge;
	}	
	
	/**	
	 * fits line y = a + bx to the data described in tables x[] and y[] 
	 */
	private double[] linear_fit (double x[],double y[]){
		double av_x = 0;
		double av_y = 0;
		double tot_xx = 0;
		double tot_xy = 0;
		double tot_yy = 0;
		double[] Values = new double[3]; // a, b, r2
		
		for (int i = 0; i < x.length; i++)
			av_x += x[i] / x.length;
		
		for (int i = 0; i < y.length; i++)
			av_y += y[i] / y.length;
		
		for (int i = 0; i < x.length; i++)
			tot_xx += Math.pow(x[i], 2);
		
		for (int i = 0; i < x.length; i++)
			tot_yy += Math.pow(y[i], 2);
		
		for (int i = 0; i < x.length; i++) //x.length = y.length
			tot_xy += x[i]*y[i];
			
		double a = 0;
		double b = 0;
		double r2 = 0;
		
		a = (av_y * tot_xx - av_x * tot_xy)   / (tot_xx - x.length * Math.pow(av_x, 2));
		b = (tot_xy - x.length * av_x * av_y) / (tot_xx - x.length * Math.pow(av_x, 2));
		r2 = Math.pow((tot_xy - x.length * av_x * av_y),2) / 
			((tot_xx - x.length * av_x * av_x)*(tot_yy - x.length*av_y*av_y));
		
		Values[0] = a;
		Values[1] = b;
		Values[2] = r2;		
		return Values;
	}	
}///:~




